function linearmodel1(q) 
% Matlab routine to replicate simulation results in the main text (Table 1--3)

R = 10000; beta = 1; n = 700;  

alpha = .990; % .995 

reject0 = zeros(R,1); cwidth0 = reject0; indbeta = zeros(R,1); indvars = zeros(R,3);
reject1 = zeros(R,1); cwidth1 = reject1;
reject2 = zeros(R,1); cwidth2 = reject2;


for r=1:R,
    n = 700;  
    w = [ones(n,1), (abs(randn(n,q-1))>norminv(alpha))]; x = zeros(n,1); y = zeros(n,1);
    
    x = randn(n,1); y = x*beta+randn;
      
    if rank(w)<q,
       constant = (mean(w)==0 | mean(w)==1); constant(1)=0; w(:,constant==1)=[]; [n q] = size(w);
       if rank(w)<q ,
           deficient=1; 
           while deficient==1,
                 A = w'*w; [V,D] = eig(A); 
                 I = (abs(V(:,1))>.0001); II = [1:1:q]; drop = max(II(I==1));
                 w(:,drop)=[]; [n q] = size(w); deficient=(rank(w)<q);
           end
       end
    end
    
    [b v0 v1 v2] = regression(y,x,w);
    indbeta(r,1) = b      ;
    indvars(r,:) = [v0 max(0,v1) max(0,v2)];
    reject0(r,1) = abs((b-beta)/sqrt(v0))>=1.96;
    reject1(r,1) = abs((b-beta)/sqrt(v1))>=1.96;
    reject2(r,1) = abs((b-beta)/sqrt(v2))>=1.96;
    cwidth0(r,1) = 2*1.96*sqrt(max(v0,0));
    cwidth1(r,1) = 2*1.96*sqrt(max(v1,0));
    cwidth2(r,1) = 2*1.96*sqrt(max(v2,0));
    
end

q,
[std(indbeta) mean(sqrt(indvars)), std(sqrt(indvars))],
[mean(reject0), mean(cwidth0), mean(reject1), mean(cwidth1), mean(reject2), mean(cwidth2)],



function [b v0 v1 v2] = regression(y,x,w)
[n q] = size(w);

a = regress(y,w); yhat = w*a;
a = regress(x,w); xhat = w*a; vhat = x-xhat; b = regress(y,vhat); ehat = (y-yhat)- (x-xhat)*b;

M = eye(n)-w*inv(w'*w)*w';  
if rank(M.*M)<size(M.*M,1), M2=eye(size(M.*M,1)); else M2 = inv(M.*M); end 

L = eye(n)./diag(M); echeck = L*ehat;

vx = mean((vhat.*vhat))  ;

vv0 = ((vhat.*vhat))'*eye(n,n)*(ehat.*ehat)/n^2; v0 = (vv0)/vx^2; % Eicker-White
vv1 = ((vhat.*vhat))'*M2*(ehat.*ehat)/n^2; v1 = (vv1)/vx^2; % CJN

perfect = (isnan(echeck)==1 | echeck==0); vhat(perfect==1)=[]; y(perfect==1)=[]; 
echeck(perfect==1)=[]; 

vv2 = mean((vhat.*vhat).*(y   .*echeck))/n; v2 = (vv2)/vx^2; % LOO

